# python version
python_inferior_to_3_7_forbidden = "you must have a version of python higher or equal to 3.7 to execute this function"

# driver
import_ogr_error = "cannot run this function ! Python-gdal library not exists"
import_psycopg2_error = "cannot run this function ! psycopg2 library not installed"

# path
path_not_valid = "path : {path} is not valid"
path_not_valid_file_exists_overwrite_is_false = "path : {path} exists."

# fields
missing_field = "field : {field_name} does not exists in geolayer"

# values

# non unique value
non_unique_values = "field : {field_name} contains non-unique values"
